package
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;
	import flash.geom.Rectangle;
	
	/**
	 * bitmapを分割してリストで返すためのクラス(BitmapData.drawを使用).
	 *
	 * 参考ページ
	 * AS3::8.画像切り出しとTweenerを使ってみる
	 * http://coelacanth.heteml.jp/site/flash_as3/article_8
	 */
	public class BmpSlice
	{
		public var orgBmp:Bitmap;
		public var bmpList:Array;
		
		public function BmpSlice(bmp:Bitmap)
		{
			orgBmp = bmp;
		}
		
		/**
		 * 分割したbitmapリストを取得する.
		 *
		 * @param	xdiv	横分割数
		 * @param	ydiv	縦分割数
		 * @return	分割画像リスト
		 */
		public function getSliceBmp(xdiv:int = 1, ydiv:int = 1):Array
		{
			// 分割画像の縦横幅を求める
			var w:Number = orgBmp.width / xdiv;
			var h:Number = orgBmp.height / ydiv;
			
			// 変換時の指定で必要になる変数を確保
			var m:Matrix = new Matrix();
			var c:ColorTransform = new ColorTransform();
			var rect:Rectangle = new Rectangle(0, 0, w, h);
			
			bmpList = new Array();
			
			for (var y:int = 0; y < ydiv; y++)
			{
				for (var x:int = 0; x < xdiv; x++)
				{
					// 分割画像用のBitmapDataを確保
					var bmpData:BitmapData = new BitmapData(w, h, true, 0xffffffff);
					
					// 分割したビットマップを描画
					bmpData.draw(orgBmp, m, c, null, rect, false);
					
					var bmp:Bitmap = new Bitmap(bmpData);
					bmp.x = w * x;
					bmp.y = h * y;
					
					bmpList.push(bmp); // リストに登録
					
					// 次に切り出す場所を指定
					if (x == xdiv - 1)
					{
						m.translate(w * (xdiv - 1), -h);
					}
					else
					{
						m.translate(-w, 0);
					}
				}
			}
			
			return bmpList;
		}
	}

}