package
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import BmpSlice;
	import BmpSliceCopyPixels;
	import MySprite;
	
	/**
	 * BitmapData のテスト
	 *
	 * 参考ページ
	 * Flashの描画速度をBitmapDataクラスを使って上げる方法 - しっぽのブログ
	 * http://sipo.jp/blog/2009/12/flashbitmapdata.html
	 */
	public class Main extends Sprite
	{
		[Embed(source="img.jpg")]
		private static const imgCls:Class;
		
		private var bitmapData:BitmapData;
		private var bitmap:Bitmap;
		private var bmp:Bitmap;
		
		private var bmpList:Array;
		private var sprList:Array;
		
		public function Main():void
		{
			if (stage)
				init();
			else
				addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private function init(e:Event = null):void
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			
			if (false)
			{
				// ビットマップ画像の表示(Embedで埋め込んだ画像を表示する例)
				bmp = new imgCls() as Bitmap;
				addChild(bmp);
			}
			
			// 分割したbitmapリストを得る
			var xd:int = 16; // 横分割数
			var yd:int = 10; // 縦分割数
			
			bmp = new imgCls() as Bitmap;
			
			if (true)
			{
				// BitmapData.draw で分割
				var bmpSlice:BmpSlice = new BmpSlice(bmp);
				bmpList = bmpSlice.getSliceBmp(xd, yd); // 分割
			}
			else
			{
				// copyPixels で分割
				var bmpSlice:BmpSliceCopyPixels = new BmpSliceCopyPixels(bmp);
				bmpList = bmpSlice.getSliceBmpList(xd, yd);
			}
			
			// 分割したbitmapリストからspriteを作成
			sprList = new Array();
			var cnt:int = bmpList.length;
			for (var i:int = 0; i < cnt; i++)
			{
				var spr:MySprite = new MySprite(bmpList[i], 0, 0, i, (xd * yd));
				addChild(spr);
				sprList.push(spr);
			}
			
			// BitmpaDataの使い方の例
			switch (3)
			{
				case 0: 
					// 基本
					// ----------------------------------------
					
					bitmapData = new BitmapData(320, 240, true, 0x8000ff00); // Bitmapdata生成
					bitmap = new Bitmap(bitmapData); // Bitmap生成
					addChild(bitmap); // 子として登録
					
					break;
				
				case 1: 
					// draw() で描画する場合
					// ----------------------------------------
					
					// とりあえずSpriteを用意
					var sprite:Sprite = new Sprite();
					
					// テスト用に円を描く
					sprite.graphics.beginFill(0xff0000);
					sprite.graphics.drawCircle(0, 0, 20);
					
					// bitmapDataとbitmapを用意する
					// 透明なBitmapDataを用意
					bitmapData = new BitmapData(200, 200, true, 0x00000000);
					bitmap = new Bitmap();
					addChild(bitmap); // 配置
					
					var matrix:Matrix = new Matrix(); // 表示位置はMatrixで指定
					matrix.translate(50, 50); // (50, 50)の位置に描画する設定にする
					bitmapData.draw(sprite, matrix); // spriteをbitmapDataに描画
					bitmap.bitmapData = bitmapData; // bitmapDataをbitmapに登録
					
					break;
				case 2: 
					// copyPixels() で描画する場合
					// ----------------------------------------
					
					// とりあえず赤色の100x100のビットマップデータを作る
					var parts:BitmapData = new BitmapData(100, 100, true, 0xffff0000);
					
					// こっちは表示用の透明なBitmapData
					bitmapData = new BitmapData(200, 200, true, 0x00000000);
					bitmap = new Bitmap();
					addChild(bitmap); // 配置
					
					var point:Point = new Point(50, 50); // 表示位置をPointで指定
					var rect:Rectangle = new Rectangle(0, 0, 100, 100); // 描画範囲をRectangleで指定
					
					bitmapData.copyPixels(parts, rect, point); // 描画
					bitmap.bitmapData = bitmapData; // bitmapDataをbitmapに登録
					break;
				case 3: 
					// 何もしない
					break;
			}
			
			addEventListener(Event.ENTER_FRAME, enterFrame); // 毎フレーム呼ばれるメソッドを登録
		}
		
		/**
		 * 毎フレーム呼ばれる処理
		 * @param	e
		 */
		public function enterFrame(e:Event):void
		{
			// sprite を動かす
			for each (var spr:MySprite in sprList) {
				spr.enterFrame();
			}
		}
	}
}